/************************************************************************
     File:        CubeIt.cpp

     Author:     
                  Michael Gleicher, gleicher@cs.wisc.edu
     Modifier
                  Yu-Chi Lai, yu-chi@cs.wisc.edu
     
     Comment:     
		 
	 					    This program makes a simple 3D world with some cubes in it.
								It provides a user interface so that the user can select cubes
								and move them around. It also provides a basic user interface
								for the camera.
								
								This program is designed to serve as a starting platform for
								CS559 projects, or as reference for how to do different things.
								Remember: if you borrow code, you must give proper attribution!
								However, you are encouraged to borrow code from this
						
			Note:   i've built this to make it easy for me to make several
							different demos, so the code organization is kindof wierd
							i've had to put lots of common stuff here that you'd normally
							put into your window/widget class

              Written at October 2005
   
     Platform:    Visio Studio.Net 2003
*************************************************************************/
#include "CubeIt.H"
#include "3DUtils.H"
#include <windows.h>
#include <GL/gl.h>

// for the Cheesy UI stuff
#include <FL/FL_Roller.H>

//*********************************************************************************
//
// Keep a list of cubes to draw, remember which one is "selected"
//*********************************************************************************
std::vector<Cube> theCubes;
int selectedCube = -1;

//*********************************************************************************
//
// Note: this is hard coded for red cubes and yellow highlights
//=================================================================================
void drawCubes(int doColors, int doNames)
//=================================================================================
{
  // note: i am going to index through the array, rather than using an
  // iterator since I want to have the index
  for(int ct=0; ct< (int) theCubes.size(); ct++) {
	  if (doColors) {
		  if (selectedCube == ct)
			  glColor3f(1,1,0);
		  else
			  glColor3f(1,0,0);
	  }
	  if (doNames)
		  glLoadName(ct + 1);	// note: we don't use the name 0!
	  drawCube(theCubes[ct].x, theCubes[ct].y, theCubes[ct].z,
			   theCubes[ct].s);
  }
}

//*********************************************************************************
//
// Cheesy UI!
// Need a callback for each dimension
// the reseting each time doesn't really work, but its a cheesy
// way to avoid having to reset the rollers when a new object is selected
//
//*********************************************************************************

//*********************************************************************************
//
// * Callback for X dimension
//=================================================================================
void xRollerCB(Fl_Widget* w, void* p)
//=================================================================================
{
  if (selectedCube > -1) {
	theCubes[selectedCube].x += (float) static_cast<Fl_Roller*>(w)->value();
	static_cast<Fl_Roller*>(w)->value(0);
	if (p) static_cast<Fl_Widget*>(p)->damage(1);
  }
}
//*********************************************************************************
//
// * Callback for Y dimension
//=================================================================================
void yRollerCB(Fl_Widget* w, void* p)
//=================================================================================
{
  if (selectedCube > -1) {
	theCubes[selectedCube].y += (float) static_cast<Fl_Roller*>(w)->value();
	static_cast<Fl_Roller*>(w)->value(0);
	if (p) static_cast<Fl_Widget*>(p)->damage(1);
  }
}
//*********************************************************************************
//
// * Callback for Z dimension
//=================================================================================
void zRollerCB(Fl_Widget* w, void* p)
//=================================================================================
{
  if (selectedCube > -1) {
	theCubes[selectedCube].z += (float) static_cast<Fl_Roller*>(w)->value();
	static_cast<Fl_Roller*>(w)->value(0);
	if (p) static_cast<Fl_Widget*>(p)->damage(1);
  }
}

//*********************************************************************************
//
// Cheesy UI!
// Need a callback for each dimension
// the reseting each time doesn't really work, but its a cheesy
// way to avoid having to reset the rollers when a new object is selected
//=================================================================================
void makeRollers(int x, int y, int h, Fl_Widget* w)
//=================================================================================
{
  Fl_Roller* rx = new Fl_Roller(x,y,25,h,"X");
  rx->minimum(-1);
  rx->maximum(1);
  rx->callback(xRollerCB,w);
  Fl_Roller* ry = new Fl_Roller(x+40,y,25,h,"Y");
  ry->minimum(-1);
  ry->maximum(1);
  ry->callback(yRollerCB,w);
  Fl_Roller* rz = new Fl_Roller(x+80,y,25,h,"Z");
  rz->minimum(-1);
  rz->maximum(1);
  rz->callback(zRollerCB,w);
}
