/************************************************************************
     File:       MyWindow.h

     Author:     
                  Steven J Chenney, schenney@cs.wisc.edu
     Modifier
                  Yu-Chi Lai, yu-chi@cs.wisc.edu
				  Justin Fiedler
     
     Comment:     Simple window class with nothing in it
				  Justin Fiedler Additions:
					A window class that handles OpenGL.  Holds a "world" with a checkerboard and
					cubes.  Various keyboard events allow user interaction:
						A: Adds a new Cube at the orgin.
						C: Toggles between moving the selected cube and moving the camera.
						S: Toggle the selected cube.
						T: Toggles the plane of movement. Default is the XZ plane, alternative is
							the XY plane.
						Arrow keys: Left/Right move in the X direction. Up/Down move in the Z/Y direction
							depending on the selected plane of movement.
						Spacebar: Animated the selected cube by sending it into a vicious spin!
   
     Platform:    Visio Studio.Net 2003
*************************************************************************/
#ifndef MY_WINDOW_H
#define MY_WINDOW_H

#include <Fl/Fl_Gl_Window.h>
#include <Fl/Fl_Button.h>
#include <vector>
#include "Cube.h"

class MyWindow : public Fl_Gl_Window
{
  public:
	//the increment of movement
	float moveInc;
	//rotation variables
	float rotation;
	float rotationIncrement;
	
	//whether or not animation is activated
	bool animating;
	//switches between XZ and XY planes
	bool planeXZ;
	//whether or not the camera is being moved
	bool cameraMove;

	//which cube is selected
	int selected;
	std::vector<Cube> cubes;
	Cube* camera;

	//Constructor and destructor
    MyWindow(int x, int y, int w, int h);
    virtual ~MyWindow();

	void InitializeGL();
	virtual void draw();
	virtual int handle(int event);
	
	//draws the cubes...
	void drawCubes(int doColors);

};

#endif
